package com.atlassian.elasticsearch.buckler;

import com.atlassian.elasticsearch.buckler.config.AuthConfig;
import com.atlassian.elasticsearch.buckler.config.BucklerConfig;
import com.atlassian.elasticsearch.buckler.security.AuthRateLimiter;
import com.atlassian.elasticsearch.buckler.security.RequestIdentifier;
import io.netty.handler.codec.http.HttpHeaderNames;
import org.apache.logging.log4j.Logger;
import org.elasticsearch.client.node.NodeClient;
import org.elasticsearch.common.logging.ESLoggerFactory;
import org.elasticsearch.rest.*;

import java.util.Collections;
import java.util.List;

class RestAuthenticationHandler implements RestHandler {

    private static final Logger log = ESLoggerFactory.getLogger(RestAuthenticationHandler.class.getName());

    private final BucklerConfig bucklerConfig;
    private final AuthRateLimiter limiter;
    private final RestHandler restHandler;

    public RestAuthenticationHandler(RestHandler restHandler, BucklerConfig bucklerConfig, AuthRateLimiter limiter) {
        this.restHandler = restHandler;
        this.bucklerConfig = bucklerConfig;
        this.limiter = limiter;
    }

    @Override
    public void handleRequest(RestRequest request, RestChannel channel, NodeClient client) throws Exception {
        if (bucklerConfig.getAuthConfig().isEnabledForHttp() && !allow(request, bucklerConfig.getAuthConfig())) {
            BytesRestResponse response = new BytesRestResponse(channel, RestStatus.UNAUTHORIZED, null);
            response.addHeader(HttpHeaderNames.WWW_AUTHENTICATE.toString(), "Basic realm=\"Restricted\"");
            channel.sendResponse(response);
            return;
        }
        restHandler.handleRequest(request, channel, client);
    }

    private boolean allow(RestRequest request, AuthConfig config) {
        List<String> authorization = request.getHeaders().getOrDefault(HttpHeaderNames.AUTHORIZATION.toString(), Collections.emptyList());
        RequestIdentifier requestIdentifier = RequestIdentifier.from(request);

        if (authorization.size() != 1) {
            return false;
        }

        if (limiter.isRequestAllowed(requestIdentifier) && config.isAuthorized(authorization.get(0))) {
            limiter.addTrustedRequest(requestIdentifier);
            return true;
        }

        if (!"".equals(authorization.get(0))) {
            log.warn("Request has been blocked due to AuthRateLimiter settings or and incorrect password from: {}",
                    request.getRemoteAddress().toString());
            limiter.registerFailure(requestIdentifier);
        }

        return false;
    }

}
